==> RoughlyEnoughItems-fabric-18.0.804-sources.jar: me/shedaniel/rei/impl/client/gui/config/components/ConfigOptionValueWidget.java <==
/*
 * This file is licensed under the MIT License, part of Roughly Enough Items.
 * Copyright (c) 2018, 2019, 2020, 2021, 2022, 2023 shedaniel
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

package me.shedaniel.rei.impl.client.gui.config.components;

import me.shedaniel.clothconfig2.api.ModifierKeyCode;
import me.shedaniel.math.Point;
import me.shedaniel.math.Rectangle;
import me.shedaniel.math.impl.PointHelper;
import me.shedaniel.rei.api.client.gui.widgets.Label;
import me.shedaniel.rei.api.client.gui.widgets.WidgetWithBounds;
import me.shedaniel.rei.api.client.gui.widgets.Widgets;
import me.shedaniel.rei.api.client.util.MatrixUtils;
import me.shedaniel.rei.api.common.util.CollectionUtils;
import me.shedaniel.rei.impl.client.gui.config.ConfigAccess;
import me.shedaniel.rei.impl.client.gui.config.REIConfigScreen;
import me.shedaniel.rei.impl.client.gui.config.options.CompositeOption;
import me.shedaniel.rei.impl.client.gui.config.options.OptionValueEntry;
import me.shedaniel.rei.impl.client.gui.modules.Menu;
import me.shedaniel.rei.impl.client.gui.modules.entries.ToggleMenuEntry;
import net.minecraft.class_124;
import net.minecraft.class_2561;
import net.minecraft.class_2960;
import net.minecraft.class_310;
import net.minecraft.class_327;
import net.minecraft.class_332;
import net.minecraft.class_437;
import net.minecraft.class_5348;
import org.joml.Matrix4f;

import java.util.Objects;
import java.util.function.BiConsumer;
import java.util.function.Consumer;

import static me.shedaniel.rei.impl.client.gui.config.options.ConfigUtils.literal;
import static me.shedaniel.rei.impl.client.gui.config.options.ConfigUtils.translatable;

public class ConfigOptionValueWidget {
    public static <T> WidgetWithBounds create(ConfigAccess access, CompositeOption<T> option, int width) {
        class_327 font = class_310.method_1551().field_1772;
        class_2561[] text = {null};
        Consumer<class_2561> setText = t -> {
            if (access.getFocusedKeycode() == option) {
                text[0] = literal("> ").method_27692(class_124.field_1054)
                        .method_10852(t.method_27661().method_27692(class_124.field_1054))
                        .method_10852(literal(" <").method_27692(class_124.field_1054));
            } else if (!(option.getEntry() instanceof OptionValueEntry.Configure<T>) && access.get(option).equals(Objects.requireNonNullElseGet(option.getDefaultValue(), () -> access.getDefault(option)))) {
                text[0] = translatable("config.rei.value.default", t);
                
                if (font.method_27525(text[0]) > width) {
                    int trimTo = width - font.method_1727("...") - (font.method_27525(text[0]) - font.method_27525(t));
                    class_5348 trimmed = font.method_1714(t, trimTo);
                    class_5348 composite = class_5348.method_29433(trimmed, literal("..."));
                    text[0] = literal(composite.getString());
                    text[0] = translatable("config.rei.value.default", text[0]);
                }
            } else if (font.method_27525(t) > width) {
                int trimTo = width - font.method_1727("...");
                class_5348 trimmed = font.method_1714(t, trimTo);
                class_5348 composite = class_5348.method_29433(trimmed, literal("..."));
                text[0] = literal(composite.getString());
            } else {
                text[0] = t;
            }
        };
        
        setText.accept(option.getEntry().getOption(access.get(option)));
        
        Matrix4f[] matrix = {new Matrix4f()};
        Label label = Widgets.createLabel(new Point(), text[0]).rightAligned()
                .color(0xFFE0E0E0)
                .hoveredColor(0xFFE0E0E0)
                .onRender((poses, l) -> {
                    if (MatrixUtils.transform(matrix[0], l.getBounds()).contains(PointHelper.ofMouse())) {
                        l.setMessage(text[0].method_27661().method_27692(class_124.field_1073));
                    } else {
                        l.setMessage(text[0]);
                    }
                });
        
        if (option.getEntry() instanceof OptionValueEntry.Selection<T> selection) {
            applySelection(access, option, selection, label, setText, matrix);
        } else if (access.get(option) instanceof ModifierKeyCode) {
            applyKeycode(access, option, label, setText, matrix);
        } else if (option.getEntry() instanceof OptionValueEntry.Configure<T>) {
            label.clickable().onClick($ -> {
                ((OptionValueEntry.Configure<T>) option.getEntry()).configure(access, option, () -> {
                    class_310.method_1551().method_1507((class_437) access);
                    setText.accept(option.getEntry().getOption(access.get(option)));
                });
            });
        }
        
        return Widgets.concatWithBounds(() -> new Rectangle(-label.getBounds().width, 0, label.getBounds().width + 8, 14),
                label,
                Widgets.createDrawableWidget((graphics, mouseX, mouseY, delta) -> matrix[0] = graphics.method_51448().method_23760().method_23761()),
                Widgets.withTranslate(Widgets.createTexturedWidget(class_2960.method_60654("roughlyenoughitems:textures/gui/config/selector.png"),
                        new Rectangle(1, 1, 4, 6), 0, 0, 1, 1, 1, 1), 0, 0.5, 0)
        );
    }
    
    private static <T> void applySelection(ConfigAccess access, CompositeOption<T> option, OptionValueEntry.Selection<T> selection, Label label, Consumer<class_2561> setText, Matrix4f[] matrix) {
        int noOfOptions = selection.getOptions().size();
        if (noOfOptions == 2) {
            label.clickable().onClick($ -> {
                access.set(option, selection.getOptions().get((selection.getOptions().indexOf(access.get(option)) + 1) % 2));
                setText.accept(selection.getOption(access.get(option)));
            });
        } else if (noOfOptions >= 2) {
            label.clickable().onClick($ -> {
                Menu menu = new Menu(MatrixUtils.transform(matrix[0], label.getBounds()), CollectionUtils.map(selection.getOptions(), opt -> {
                    class_2561 selectionOption = selection.getOption(opt);
                    if (opt.equals(access.getDefault(option))) {
                        selectionOption = translatable("config.rei.value.default", selectionOption);
                    }
                    
                    return ToggleMenuEntry.of(selectionOption, () -> false, o -> {
                        ((REIConfigScreen) class_310.method_1551().field_1755).closeMenu();
                        access.set(option, opt);
                        setText.accept(selection.getOption(opt));
                    });
                }), false);
                access.closeMenu();
                access.openMenu(menu);
            });
        }
    }
    
    private static <T> void applyKeycode(ConfigAccess access, CompositeOption<T> option, Label label, Consumer<class_2561> setText, Matrix4f[] matrix) {
        label.clickable().onClick($ -> {
            access.closeMenu();
            access.focusKeycode((CompositeOption<ModifierKeyCode>) option);
        });
        BiConsumer<class_332, Label> render = label.getOnRender();
        label.onRender((poses, $) -> {
            render.accept(poses, $);
            setText.accept(((ModifierKeyCode) access.get(option)).getLocalizedName());
        });
    }
}

==> RoughlyEnoughItems-fabric-19.0.806-sources.jar: me/shedaniel/rei/impl/client/gui/config/components/ConfigOptionValueWidget.java <==
/*
 * This file is licensed under the MIT License, part of Roughly Enough Items.
 * Copyright (c) 2018, 2019, 2020, 2021, 2022, 2023 shedaniel
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

package me.shedaniel.rei.impl.client.gui.config.components;

import me.shedaniel.clothconfig2.api.ModifierKeyCode;
import me.shedaniel.math.Point;
import me.shedaniel.math.Rectangle;
import me.shedaniel.math.impl.PointHelper;
import me.shedaniel.rei.api.client.gui.widgets.Label;
import me.shedaniel.rei.api.client.gui.widgets.WidgetWithBounds;
import me.shedaniel.rei.api.client.gui.widgets.Widgets;
import me.shedaniel.rei.api.client.util.MatrixUtils;
import me.shedaniel.rei.api.common.util.CollectionUtils;
import me.shedaniel.rei.impl.client.gui.config.ConfigAccess;
import me.shedaniel.rei.impl.client.gui.config.REIConfigScreen;
import me.shedaniel.rei.impl.client.gui.config.options.CompositeOption;
import me.shedaniel.rei.impl.client.gui.config.options.OptionValueEntry;
import me.shedaniel.rei.impl.client.gui.modules.Menu;
import me.shedaniel.rei.impl.client.gui.modules.entries.ToggleMenuEntry;
import net.minecraft.class_124;
import net.minecraft.class_2561;
import net.minecraft.class_2960;
import net.minecraft.class_310;
import net.minecraft.class_327;
import net.minecraft.class_332;
import net.minecraft.class_437;
import net.minecraft.class_5348;
import java.util.Objects;
import java.util.function.BiConsumer;
import java.util.function.Consumer;

import static me.shedaniel.rei.impl.client.gui.config.options.ConfigUtils.literal;
import static me.shedaniel.rei.impl.client.gui.config.options.ConfigUtils.translatable;

public class ConfigOptionValueWidget {
    public static <T> WidgetWithBounds create(ConfigAccess access, CompositeOption<T> option, int width) {
        class_327 font = class_310.method_1551().field_1772;
        class_2561[] text = {null};
        Consumer<class_2561> setText = t -> {
            if (access.getFocusedKeycode() == option) {
                text[0] = literal("> ").method_27692(class_124.field_1054)
                        .method_10852(t.method_27661().method_27692(class_124.field_1054))
                        .method_10852(literal(" <").method_27692(class_124.field_1054));
            } else if (!(option.getEntry() instanceof OptionValueEntry.Configure<T>) && access.get(option).equals(Objects.requireNonNullElseGet(option.getDefaultValue(), () -> access.getDefault(option)))) {
                text[0] = translatable("config.rei.value.default", t);
                
                if (font.method_27525(text[0]) > width) {
                    int trimTo = width - font.method_1727("...") - (font.method_27525(text[0]) - font.method_27525(t));
                    class_5348 trimmed = font.method_1714(t, trimTo);
                    class_5348 composite = class_5348.method_29433(trimmed, literal("..."));
                    text[0] = literal(composite.getString());
                    text[0] = translatable("config.rei.value.default", text[0]);
                }
            } else if (font.method_27525(t) > width) {
                int trimTo = width - font.method_1727("...");
                class_5348 trimmed = font.method_1714(t, trimTo);
                class_5348 composite = class_5348.method_29433(trimmed, literal("..."));
                text[0] = literal(composite.getString());
            } else {
                text[0] = t;
            }
        };
        
        setText.accept(option.getEntry().getOption(access.get(option)));
        
        Label label = Widgets.createLabel(new Point(), text[0]).rightAligned()
                .color(0xFFE0E0E0)
                .hoveredColor(0xFFE0E0E0)
                .onRender((poses, l) -> {
                    if (MatrixUtils.transform(poses.method_51448().method_23760().method_23761(), l.getBounds()).contains(PointHelper.ofMouse())) {
                        l.setMessage(text[0].method_27661().method_27692(class_124.field_1073));
                    } else {
                        l.setMessage(text[0]);
                    }
                });
        
        if (option.getEntry() instanceof OptionValueEntry.Selection<T> selection) {
            applySelection(access, option, selection, label, setText);
        } else if (access.get(option) instanceof ModifierKeyCode) {
            applyKeycode(access, option, label, setText);
        } else if (option.getEntry() instanceof OptionValueEntry.Configure<T>) {
            label.clickable().onClick($ -> {
                ((OptionValueEntry.Configure<T>) option.getEntry()).configure(access, option, () -> {
                    class_310.method_1551().method_1507((class_437) access);
                    setText.accept(option.getEntry().getOption(access.get(option)));
                });
            });
        }
        
        return Widgets.concatWithBounds(() -> new Rectangle(-label.getBounds().width, 0, label.getBounds().width + 8, 14),
                label,
                Widgets.withTranslate(Widgets.createTexturedWidget(class_2960.method_60654("roughlyenoughitems:textures/gui/config/selector.png"),
                        new Rectangle(1, 1, 4, 6), 0, 0, 1, 1, 1, 1), 0, 0.5, 0)
        );
    }
    
    private static <T> void applySelection(ConfigAccess access, CompositeOption<T> option, OptionValueEntry.Selection<T> selection, Label label, Consumer<class_2561> setText) {
        Rectangle bounds = new Rectangle();
        BiConsumer<class_332, Label> render = label.getOnRender();
        label.onRender((poses, $) -> {
            render.accept(poses, $);
            bounds.setBounds(MatrixUtils.transform(poses.method_51448().method_23760().method_23761(), label.getBounds()));
        });
        int noOfOptions = selection.getOptions().size();
        if (noOfOptions == 2) {
            label.clickable().onClick($ -> {
                access.set(option, selection.getOptions().get((selection.getOptions().indexOf(access.get(option)) + 1) % 2));
                setText.accept(selection.getOption(access.get(option)));
            });
        } else if (noOfOptions >= 2) {
            label.clickable().onClick($ -> {
                Menu menu = new Menu(bounds, CollectionUtils.map(selection.getOptions(), opt -> {
                    class_2561 selectionOption = selection.getOption(opt);
                    if (opt.equals(access.getDefault(option))) {
                        selectionOption = translatable("config.rei.value.default", selectionOption);
                    }
                    
                    return ToggleMenuEntry.of(selectionOption, () -> false, o -> {
                        ((REIConfigScreen) class_310.method_1551().field_1755).closeMenu();
                        access.set(option, opt);
                        setText.accept(selection.getOption(opt));
                    });
                }), false);
                access.closeMenu();
                access.openMenu(menu);
            });
        }
    }
    
    private static <T> void applyKeycode(ConfigAccess access, CompositeOption<T> option, Label label, Consumer<class_2561> setText) {
        label.clickable().onClick($ -> {
            access.closeMenu();
            access.focusKeycode((CompositeOption<ModifierKeyCode>) option);
        });
        BiConsumer<class_332, Label> render = label.getOnRender();
        label.onRender((poses, $) -> {
            render.accept(poses, $);
            setText.accept(((ModifierKeyCode) access.get(option)).getLocalizedName());
        });
    }
}
